/*
 * Copyright (C) 2013 Deepin, Inc.
 *                    Zhai Xiang <zhaixiang@linuxdeepin.com>
 *
 * Please read the source code of xf86-input-evdev about the mtdev
 *
 */

#include <iostream>
#include <fstream>
#include <stdio.h>
#include <signal.h>
#include <stdlib.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <unistd.h>
#include <mtdev.h>

#include "timer.h"

#define DEVICE "/dev/input/event6"
#define NUM_EVENTS 16 /* just a magic number to reduce the number of reads */
#define SAMPLE_FILE "sample.txt"
#define CHECK(dev, name)            \
    if (mtdev_has_mt_event(dev, name))  \
        std::cout << "support " << #name << std::endl;  \
    else    \
        std::cout << "unsupport " << #name << std::endl;

typedef struct {
    int slot;
    int x;
    int y;
} point_t;

enum SlotState {
    SLOTSTATE_OPEN = 8, /* Touch Begin */
    SLOTSTATE_CLOSE,    /* Touch Close */
    SLOTSTATE_UPDATE,   /* Touch Update */
    SLOTSTATE_EMPTY,
};

static struct mtdev* m_dev = NULL;
static int m_fd = -1;
static enum SlotState m_slot_state = SLOTSTATE_EMPTY;
static std::ofstream m_sample_file;
static int m_slot = 0;
static int m_pos_x = -1;
static int m_pos_y = -1;
static bool m_is_read = false;

static void m_process_absolute_motion_event(struct input_event* ev) 
{
    int value = ev->value;

    /*printf("%02x\n", ev->code);*/

    if (ev->code > ABS_MAX) 
        return;
  
    if (ev->code == ABS_MT_SLOT) 
    {
        m_slot = value;
    } 
    else 
    {
        m_slot_state = SLOTSTATE_UPDATE;
    }

    if (ev->code == ABS_MT_POSITION_X) 
    {
        m_pos_x = value;
    }

    if (ev->code == ABS_MT_POSITION_Y)
    {
        m_pos_y = value;
    }

    if (ev->code == ABS_MT_TRACKING_ID)
    {
        if (value >= 0) 
            m_slot_state = SLOTSTATE_OPEN;
        else 
            m_slot_state = SLOTSTATE_CLOSE;
    }

    printf("slot[%d] (x, y) = (%d, %d)\n", m_slot, m_pos_x, m_pos_y);
    m_sample_file << m_slot << " " << m_pos_x << " " << m_pos_y << "\n";
}

static void m_process_event(struct input_event* ev) 
{
    switch (ev->type) 
    {
        case EV_ABS:
            m_process_absolute_motion_event(ev);
            break;
    }
}

static void m_cleanup() 
{
    m_sample_file.close();

    if (m_dev) 
    {
        mtdev_close_delete(m_dev);
        m_dev = NULL;
    }

    if (m_fd) 
    {
        close(m_fd);
        m_fd = -1;
    }
}

static void m_signal_callback_handler(int signum) 
{
    std::cout << "Bye :)" << std::endl;
    m_cleanup();
    exit(signum);
}

static void m_timer_callback() 
{
    std::cout << "DEBUG: timer is active" << std::endl;
}

int main(int argc, char* argv[]) 
{
    struct input_event ev[NUM_EVENTS];
    int i, len = sizeof(ev);

    if (argc < 2)
        m_fd = open(DEVICE, O_RDONLY);
    else
        m_fd = open(argv[1], O_RDONLY);
    if (m_fd == -1) 
    {
        std::cout << "fail to open device " << DEVICE << std::endl;
        return -1;
    }

    if (m_dev = mtdev_new_open(m_fd)) 
    {
        unlink(SAMPLE_FILE);
        m_sample_file.open(SAMPLE_FILE);
        
        CHECK(m_dev, ABS_MT_SLOT);
        CHECK(m_dev, ABS_MT_TOUCH_MAJOR);
        CHECK(m_dev, ABS_MT_TOUCH_MINOR);
        CHECK(m_dev, ABS_MT_WIDTH_MAJOR);
        CHECK(m_dev, ABS_MT_WIDTH_MINOR);
        CHECK(m_dev, ABS_MT_ORIENTATION);
        CHECK(m_dev, ABS_MT_POSITION_X);
        CHECK(m_dev, ABS_MT_POSITION_Y);
        CHECK(m_dev, ABS_MT_TOOL_TYPE);
        CHECK(m_dev, ABS_MT_BLOB_ID);
        CHECK(m_dev, ABS_MT_TRACKING_ID);
        CHECK(m_dev, ABS_MT_PRESSURE);
        CHECK(m_dev, ABS_MT_DISTANCE);
        CHECK(m_dev, ABS_MT_TOOL_X);
        CHECK(m_dev, ABS_MT_TOOL_Y);
        
        std::cout << "ctrl+c to quit" << std::endl;
        signal(SIGINT, m_signal_callback_handler);

        while (len == sizeof(ev)) 
        {
            dmt::Timer timer(m_timer_callback);
            len = mtdev_get(m_dev, m_fd, ev, NUM_EVENTS) * sizeof(struct input_event);
            if (len <= 0 || len % sizeof(ev[0])) 
            {
                std::cout << "read error" << std::endl;
            }

            for (i = 0; i < len / sizeof(ev[0]); i++) 
            {
                m_process_event(&ev[i]);
            }
        }
    }

    return 0;
}
