#include <iostream>
#include <fstream>                                                              
#include <set>                                                                  
#include <vector>
#include <stdio.h>
#include <stdlib.h>
#include <vtkVersion.h>
#include <vtkSmartPointer.h>
#include <vtkAxis.h>
#include <vtkChartXY.h>
#include <vtkContextScene.h>
#include <vtkContextView.h>
#include <vtkFloatArray.h>
#include <vtkPlotPoints.h>
#include <vtkRenderWindow.h>
#include <vtkRenderWindowInteractor.h>
#include <vtkRenderer.h>
#include <vtkTable.h>

#define BUFFER_SIZE 1024

typedef struct {                                                                
    int x;                                                                      
    int y;                                                                      
} point_t;                                                                      
                                                                                
typedef struct {                                                                
    int slot;                                                                   
    point_t point;                                                              
} sample_t;

static void m_multi_slots(std::set<int> slots, 
                          std::vector<sample_t> samples, 
                          int only_shown_slot);

int main(int argc, char* argv[])
{
    std::ifstream file;                                                         
    int slot;                                                                   
    int x, y;                                                                   
    int only_shown_slot = argv[2] ? atoi(argv[2]) : -1;
    std::vector<sample_t> samples;                                              
    std::vector<sample_t>::iterator sample_iter;                                
    std::set<int> slots;

    if (argc < 2)                                                               
    {                                                                           
        std::cout << "usage: <sample file> only_shown_slot" << std::endl;                       
        return -1;                                                              
    }

    file.open(argv[1]);                                                         
    while (file >> slot >> x >> y)                                              
    {                                                                           
        if (x < 0 || y < 0)                                                     
            continue;                                                           
        point_t point;                                                          
        point.x = x;                                                            
        point.y = y;                                                            
        sample_t sample;                                                        
        sample.slot = slot;                                                     
        sample.point = point;                                                   
        samples.push_back(sample);                                              
    }                                                                           
    file.close();

    std::cout << "DEBUG: sample size " << samples.size()  << std::endl;         
    for (sample_iter = samples.begin(); sample_iter < samples.end(); sample_iter++)
    {                                                                           
        slots.insert((*sample_iter).slot);                                      
    }                                                                           
    std::cout << "DEBUG: slot size " << slots.size() << std::endl;              
    if (slots.size() < 1)                                                       
        return -1;

    if (only_shown_slot != -1 && slots.size() - 1 < only_shown_slot) 
    {
        printf("DEBUG: only_shown_slot %d is out of range %d\n", 
                only_shown_slot, slots.size());
        return -1;
    }

    m_multi_slots(slots, samples, only_shown_slot);

    return 0;
} 

static vtkSmartPointer<vtkTable> m_create_table(int slot, std::vector<point_t> points) 
{
    vtkSmartPointer<vtkTable> table = vtkSmartPointer<vtkTable>::New();
    vtkSmartPointer<vtkFloatArray> arrX = vtkSmartPointer<vtkFloatArray>::New();
    vtkSmartPointer<vtkFloatArray> arrY = vtkSmartPointer<vtkFloatArray>::New();
    std::vector<point_t>::iterator iter;
    char buffer[BUFFER_SIZE];
    int i;

    arrX->SetName("X Axis");
    table->AddColumn(arrX);

    memset(buffer, 0, BUFFER_SIZE);
    snprintf(buffer, BUFFER_SIZE, "Slot_%d", slot);
    arrY->SetName(buffer);
    table->AddColumn(arrY);

    table->SetNumberOfRows(points.size());

    for (i = 0, iter = points.begin(); iter != points.end(); i++, iter++) 
    {                                                                           
        table->SetValue(i, 0, (*iter).x);                                 
        table->SetValue(i, 1, (*iter).y);                                 
    }

    return table;
}

static void m_multi_slots(std::set<int> slots, 
                          std::vector<sample_t> samples, 
                          int only_shown_slot) 
{
    std::set<int>::iterator slot_iter;                                          
    std::vector<sample_t>::iterator sample_iter;
    std::vector<point_t>::iterator point_iter;    
    std::vector<point_t>* slot_points = new std::vector<point_t>[slots.size()];
    vtkPlot* points = NULL;
    int i;
    
    // Set up a 2D scene, add an XY chart to it
    vtkSmartPointer<vtkContextView> view = vtkSmartPointer<vtkContextView>::New();
    view->GetRenderer()->SetBackground(1.0, 1.0, 1.0);
    view->GetRenderWindow()->SetSize(800, 600);

    vtkSmartPointer<vtkChartXY> chart = vtkSmartPointer<vtkChartXY>::New();
    view->GetScene()->AddItem(chart);
    chart->SetShowLegend(true);

    // Create a table with some points in it...
    vtkSmartPointer<vtkTable> table = vtkSmartPointer<vtkTable>::New();

    vtkSmartPointer<vtkFloatArray> arrX = vtkSmartPointer<vtkFloatArray>::New();
    arrX->SetName("X Axis");
    table->AddColumn(arrX);

    vtkSmartPointer<vtkFloatArray> arrT = vtkSmartPointer<vtkFloatArray>::New();
    arrT->SetName("T Axis");                                                    
    table->AddColumn(arrT);

    for (slot_iter = slots.begin(); slot_iter != slots.end(); slot_iter++)      
    {                                                                           
        for (sample_iter = samples.begin(); sample_iter != samples.end(); sample_iter++)
        {                                                                       
            if (*slot_iter == (*sample_iter).slot)                              
            {                                                                   
                point_t point;                                                  
                point.x = (*sample_iter).point.x;                               
                point.y = (*sample_iter).point.y;                               
                slot_points[*slot_iter].push_back(point);                       
            }                                                                   
        }
        printf("DEBUG: slot_points[%d] size %d\n", 
               *slot_iter, slot_points[*slot_iter].size());
    }

    for (slot_iter = slots.begin(); slot_iter != slots.end(); slot_iter++)         
    {                                          
        if (only_shown_slot != -1) 
        {
            if (only_shown_slot != *slot_iter) 
                continue;
        }
        points = chart->AddPlot(vtkChart::POINTS);                      
        points->SetInput(m_create_table(*slot_iter, slot_points[*slot_iter]), 0, 1);
        switch (*slot_iter) 
        {
        case 4:
            points->SetColor(255, 255, 0, 255);
            break;
        case 3:
            points->SetColor(255, 0, 255, 255);
            break;
        case 2:
            points->SetColor(0, 255, 0, 255);
            break;
        case 1:
            points->SetColor(255, 0, 0, 255);
            break;
        case 0:
        default:
            points->SetColor(0, 0, 255, 255);
        }
        points->SetWidth(1.0);                                              
        vtkPlotPoints::SafeDownCast(points)->SetMarkerStyle(vtkPlotPoints::CIRCLE);
    }
    // Y Axis
    chart->GetAxis(0)->SetBehavior(vtkAxis::FIXED);
    chart->GetAxis(0)->SetRange(0, 2050);
    // X Axis
    chart->GetAxis(1)->SetBehavior(vtkAxis::FIXED);
    chart->GetAxis(1)->SetRange(0, 4000);

    //Finally render the scene
    view->GetRenderWindow()->SetMultiSamples(0);
    view->GetInteractor()->Initialize();
    view->GetInteractor()->Start();
  
    delete [] slot_points;                                                      
    slot_points = NULL;
}
